# Tests for student related reports
import os
from zc.async.testing import wait_for_result
from zope.interface.verify import verifyClass, verifyObject
from zope.component import getUtility
from waeup.kofa.interfaces import IJobManager
from waeup.kofa.students.tests.test_catalog import CatalogTestSetup
from waeup.kofa.students.tests.test_browser import StudentsFullSetup
from waeup.kofa.tests.test_async import FunctionalAsyncTestCase
from waeup.kofa.browser.tests.test_pdf import samples_dir

from waeup.aaue.testing import FunctionalLayer
from waeup.aaue.students.reports.student_level_statistics import (
    get_student_stats, StudentLevelStatisticsReport, IStudentLevelStatisticsReport)

class StudentLevelStatisticsReportTests(CatalogTestSetup):

    layer = FunctionalLayer

    levels = (u'100 male', u'100 female', u'200 male', u'200 female', u'300 male', u'300 female',
              u'400 male', u'400 female', u'500 male', u'500 female', u'Total male', u'Total female')

    def test_iface(self):
        # ensure we fullfill interface contracts
        obj = StudentLevelStatisticsReport(2010, 'faccode')
        verifyClass(IStudentLevelStatisticsReport, StudentLevelStatisticsReport)
        verifyObject(IStudentLevelStatisticsReport, obj)
        return

    def test_get_student_stats_session_simple(self):
        self.student['studycourse'].current_level = 110
        self.student.sex = 'm'
        # we can get a table with one student
        result1 = get_student_stats(2010, 'faccode')
        result2 = get_student_stats(2009, 'faccode')
        self.assertEqual(
            result1,
            ((u'fac1', u'Total',),
             self.levels,
             ((1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0),
              (1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0))
            ))
        self.assertEqual(
            result2,
            ((u'fac1', u'Total'),
             self.levels,
             ((0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0),
              (0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0))
            ))
        return

    def test_get_student_stats_session_multiple(self):
        # we can get a table with several faculties
        self.create_cert(u'fac2', u'dep2', u'CERT2')
        self.student.sex = 'm'
        result1 = get_student_stats(2010, 'faccode')
        result2 = get_student_stats(2009, 'faccode')
        self.assertEqual(
            result1,
            ((u'fac1', u'fac2', u'Total'),
             self.levels,
             ((1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0),
              (0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0),
              (1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0))
            ))
        self.assertEqual(
            result2,
            ((u'fac1', u'fac2', u'Total'),
             self.levels,
             ((0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0),
              (0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0),
              (0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0))
            ))
        return

    def test_get_student_stats_session_multiple_dep_breakdown(self):
        # we can get a table with several students
        self.create_cert(u'fac2', u'dep2', u'CERT2')
        self.student.sex = 'm'
        result1 = get_student_stats(2010, 'depcode')
        result2 = get_student_stats(2009, 'depcode')
        self.assertEqual(
            result1,
            ((u'fac1/dep1', u'fac2/dep2', u'Total'),
             self.levels,
             ((1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0),
              (0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0),
              (1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0))
             ))
        self.assertEqual(
            result2,
            ((u'fac1/dep1', u'fac2/dep2', u'Total'),
             self.levels,
             ((0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0),
              (0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0),
              (0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0))
             ))
        return

    def test_create_pdf(self):
        self.student.sex = 'm'
        self.create_cert(u'FAC2', u'dept2', u'CERT2')
        self.student['studycourse'].current_level = 220
        report = StudentLevelStatisticsReport(2010, 'depcode')
        result = report.create_pdf()
        self.assertTrue(result.startswith('%PDF-'))
        path = os.path.join(samples_dir(), 'student_level_statistics.pdf')
        open(path, 'wb').write(result)
        print "Sample PDF student_level_statistics.pdf written to %s" % path
        return
